/* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * By using this code and algorithm, you agree that you are completely
 * responisble for your own actions. LAN tools project, and I, Nickolsky Artyom, are in no way
 * liable for any actions taken using this code and algorithm.
 *
 *  Copyright 2001 LAN tools project. 
 *  Copyright 2001 Nickolsky Artyom.
 * E-mail: nickolsky@yandex.ru
 * URL: http://lantools.narod.ru
 */
#ifndef ___PROTOCOL_H_
#define ___PROTOCOL_H_

#pragma pack( push, 1 )

struct SEthernetPacket
{
	unsigned char bDest[ 6 ];
	unsigned char bSource[ 6 ];
	WORD wDataSize;
};

struct SIPXAddress
{
	unsigned char bNetwork[ 4 ];
	unsigned char bNode[ 6 ];
	WORD wSocket;
};

#define IPX_SOCKET_MAILSLOT 0x5305

struct SIPXPacket
{
	WORD wCheckSum;
	WORD wIDPLength;
	unsigned char bTransportControl;
	unsigned char bPacketType;
	SIPXAddress Dest;
	SIPXAddress Source;
};

#define IPX_TYPE_WANBROADCAST 0x14

struct SNMPIPacket
{
	DWORD dwRoutingInfo[ 8 ];
	unsigned char bPacketType;
	unsigned char bNameType;
	WORD wMessageId;
	char sDestName[ 16 ];
	char sSourceName[ 16 ];
};

#define NMPI_TYPE_MAILSLOTWRITE 0xFC
#define NMPI_NAME_MACHINE 0x01
#define NMPI_NAME_WORKGROUP 0x02


struct SSMBStatus
{
	unsigned char bErrorClass;
	unsigned char bData;
	WORD wErrorCode;
};

struct SSMBPacket
{
	unsigned char bType;
	char sTitle[ 3 ];
	unsigned char bCommand;
	SSMBStatus Status;
	unsigned char bFlags;
	WORD wFlags2;
	unsigned char bData[ 12 ];
	WORD wTreeId;
	WORD wProcessId;
	WORD wUserId;
	WORD wMultiplexId;
};

#define SMB_COMMAND_CTRANSACT 0x25
#define SMB_COMMAND_CSENDMESSAGE 0xD0
#define SMB_COMMAND_CBEGINMESSAGE 0xD5
#define SMB_COMMAND_CENDMESSAGE 0xD6
#define SMB_COMMAND_CMESSAGETEXT 0xD7
#define SMB_COMMAND_CTREECONNECT 0x75
#define SMB_COMMAND_CNEGOTIATE 0x72

struct SCTransactPacket
{
	unsigned char bWordCount;
	WORD wTotalParmBytes;
	WORD wTotalDataBytes;
	WORD wMaxParmBytes;
	WORD wMaxDataBytes;
	unsigned char bMaxSetupWords;
	unsigned char bData;
	WORD wTransactFlagsSummary;
	DWORD dwTransactTimeOut;
	WORD wData;
	WORD wParamBytes;
	WORD wParamOffset;
	WORD wDataBytes;
	WORD wDataOffset;
	unsigned char bMaxSetupWords2;
	unsigned char bData2;
	WORD wMailSlotOpCode;
	WORD wTransactionPriority;
	WORD wMailSlotClass;
	WORD wByteCount;
};

struct SIPAddress
{
	unsigned char bFirst;
	unsigned char bSecond;
	unsigned char bThird;
	unsigned char bFourth;
};

struct SIPPacket
{
	char bVersionAndHeader;
	char bServiceType;
	WORD wTotalLength;
	WORD wIdentification;
	WORD wFlagsAndFragmentOffset;
	char bTimeToLive;
	char bProtocol; //Underlying protocol;
	WORD wChecksum;
	SIPAddress SourceAddress;
	SIPAddress DestAddress;
};

#define UDP_PROTOCOL_NUMBER 0x11
#define TCP_PROTOCOL_NUMBER 0x6

struct SUDPPacket
{
	WORD wSourcePort;
	WORD wDestPort;
	WORD wTotalLength;
	WORD wUDPChecksum;
};

#define NETBIOS_DGRAM_PORT 0x8A00
#define NETBIOS_SESSION_PORT 0x8B

struct SNBTPacketDU
{	
	char bDatagramPacketType;
	char bFlags;
	WORD wDatagramId;
	SIPAddress SourceIP;
	WORD wSourcePort;
	WORD wDatagramLength;
	WORD wPacketOffset;
	char sSourceName[ 34 ];
	char sDestName[ 34 ];
};

struct SNBTPacketSM
{
	char bPacketType;
	char bFlags;
	WORD wPacketLength;
};

#define NBT_DIRECT_UNIQUE 0x10
#define NBT_DIRECT_GROUP 0x11
#define NBT_SESSION_MESSAGE 0x00
#define NBT_SESSION_REQUEST 0x81
#define NBT_SESSION_SUCCESS 0x82

struct sNBTNewSessionPacket
{
	SNBTPacketSM header;
	char calledName[34];
	char callingName[34];
};

struct SSMBTreeConnectHeader
{
	unsigned char bWordCount;
	WORD wParam;
	WORD wNextOffset;
	WORD wDisconnectFlag;
	WORD wPasswordLength;
	WORD wByteCount;
};

struct SSMBTreeConnect
{
	SNBTPacketSM nbtHeader;
	SSMBPacket smbHeader;
	SSMBTreeConnectHeader treeHeader;
	char sData[1500];
};

struct SSMBNegotiateHeader
{
	unsigned char bWordCount;
	WORD wByteCount;
};

struct SSMBNegotiate
{
	SNBTPacketSM nbtHeader;
	SSMBPacket smbHeader;
	SSMBNegotiateHeader negotiateHeader;
	char sData[1500];
};

struct SCSendMessagePacket
{
	unsigned char bWordCount;
	char bId;
};

struct STCPPacket
{
	WORD wSourcePort;
	WORD wDestPort;
	DWORD dwSeqNumber;
	DWORD dwAcknNumber;
	WORD wFlags;
	WORD wWindow;
	WORD wCheckSum;
	WORD wUrgentPtr;
};

struct SIPXMailslotPacket
{
	SEthernetPacket ether;
	SIPXPacket ipx;
	SNMPIPacket nmpi;
	SSMBPacket smb;
	SCTransactPacket tr;
	char data[ 2 ];
};

struct SIPXNMBSPacket
{
	SEthernetPacket ether;
	SIPXPacket ipx;
	SSMBPacket smb;
	SCSendMessagePacket sm;
	char data[ 2 ];
};

struct SIPMailslotPacket
{
	SEthernetPacket ether;
	SIPPacket ip;
	SUDPPacket udp;
	SNBTPacketDU nbt;
	SSMBPacket smb;
	SCTransactPacket tr;
	char data[ 2 ];
};

struct SIPNMBSPacket
{
	SEthernetPacket ether;
	SIPPacket ip;
	STCPPacket tcp;
	SNBTPacketSM nbt;
	SSMBPacket smb;
	SCSendMessagePacket sm;
	char data[ 2 ];
};


#pragma pack( pop )

#endif